import QtQuick 2.6
import QtQuick.Controls 1.5
import QtQuick.Layouts 1.9

import "pages" as Page

import "execStage.js" as ExecutionStage
import "strings.js" as Locales
import "siteMap.js" as SiteMap

Rectangle {

	id: nag
	color: "#131418"
	focus: true
	border.width: 1
	border.color: "#293242"
	width: 677
	property var strings
	property var availableSites
	property int currentStage
	readonly property int lowerGoodRating: 6

	property int userNPS: 0
	property string userFeedback: ""
	property string userEmail: ""
	property bool isFeedbackAndNPSSent: false

	readonly property var stages: new Map()
		.set(ExecutionStage.chooseFeedback, {
			pageIndex: 0,
			height: 382,
			width: 677,
			submit: function() {
				onNPSSubmitted()
			},
			activate: function() {
				ratePage.activate()
			},
		})

		.set(ExecutionStage.enterBadComment, {
			pageIndex: 1,
			height: 507,
			width: 558,
			submit: function() {
				onFeedbackSubmitted(badCommentPage.commentText, badCommentPage.emailText)
			},
		})

		.set(ExecutionStage.displaySiteButtons, {
			pageIndex: 2,
			width: 558,
			height: (availableSites.length < 4) ? 303 : 368
		})

		.set(ExecutionStage.enterGoodComment, {
			pageIndex: 3,
			height: 464,
			width: 558,
			submit: function() {
				onFeedbackSubmitted(goodCommentPage.commentText, goodCommentPage.emailText)
			}
		})

		.set(ExecutionStage.finalPage, {
			pageIndex: 4,
			height: 222,
			width: 447,
		})

	StackLayout {
		id: content
		anchors.fill: parent

		Page.RankPage {
			id: ratePage
			npsTitleText     : strings.RATE_TITLE
			worstFeedbackText: strings.WORST_FEEDBACK
			bestFeedbackText : strings.BEST_FEEDBACK
			submitButtonText : strings.SUBMIT_TEXT
			submitEnabled    : false
			onCloseRequested : nag.sendFeedbackAndClose()
			onSubmitClicked  : nag.submit()
		}

		Page.BadComments {
			id: badCommentPage
			messageSentText   : strings.MESSAGE_SENT
			badTitleText      : strings.FB_REQUEST
			badSubTitleText   : strings.CONTACT_SUPPORT
			sendButtonText    : strings.SUBMIT_TEXT
			messageTitle      : strings.MESSAGE_TITLE
			messagePlaceholder: strings.MESSAGE_PLACEHOLDER
			emailPlaceholder  : strings.EMAIL_PLACEHOLDER
			emailTitle        : strings.EMAIL_TITLE
			invalidEmail      : strings.WRONG_EMAIL
			privacyPolicyText : strings.PRIVACY_POLICY
			bottomLinkText    : "<a href=\"#\">" + strings.LATER_TEXT + "</a>"
			onButtonClicked   : nag.submit()
			onLinkClicked     : nag.activateBottomLink()
			onCloseRequested  : nag.sendFeedbackAndClose()
		}

		Page.SitesPage {
			id: siteButtonsPage
			sites              : nag.availableSites
			messageSentText    : strings.MESSAGE_SENT
			siteButtonsTitle   : strings.FB_SITE_REQUEST
			bottomLinkText     : "<a href=\"#\">" + strings.SEND_US_FB + "</a>"
			onBottomLinkClicked: nag.activateBottomLink()
			onCloseRequested   : nag.sendFeedbackAndClose()
			onSiteButtonClicked: function(siteName) {
				callback.RequestUrl("http://links.movavi.com/leave-feedback/?asrc=npsnag&-TAIL_WITH_ARGS-&target_url=" + siteName)
			}
		}

		Page.GoodComments {
			id: goodCommentPage
			goodTitleText      : strings.FB_REQUEST_GOOD
			sendButtonText     : strings.SUBMIT_TEXT
			messageTitle       : strings.MESSAGE_TITLE
			messagePlaceholder : strings.MESSAGE_PLACEHOLDER
			emailPlaceholder   : strings.EMAIL_PLACEHOLDER
			privacyPolicyText  : strings.PRIVACY_POLICY
			emailTitle         : strings.EMAIL_TITLE
			invalidEmail       : strings.WRONG_EMAIL
			bottomLinkText     : "<a href=\"#\">" + strings.LATER_TEXT + "</a>"
			onBottomLinkClicked: nag.activateBottomLink()
			onCloseRequested   : nag.sendFeedbackAndClose()
			onButtonClicked    : nag.submit()
		}

		Page.ThankYou {
			id: finalPage
			titleText       : strings.FINAL_TEXT
			closeButtonText : strings.CLOSE_TEXT
			onCloseClicked  : {
				callback.Accept()
				nag.reset()
			}
		}
	}

	Component.onCompleted: {
		strings = Locales.stringList[callback.GetLocale()]
		availableSites = SiteMap.stringList[callback.GetLocale()]

		activatePage(ExecutionStage.chooseFeedback)
	}

	Keys.onEscapePressed: sendFeedbackAndClose()

	function submit() {
		var stage = stages.get(currentStage)
		if (stage.submit)
			stage.submit()
	}

	function activateBottomLink() {
		if (currentStage == ExecutionStage.displaySiteButtons)
			activatePage(ExecutionStage.enterGoodComment)
		else
			sendFeedbackAndClose()
	}

	function activatePage(executionStage) {
		var stage = stages.get(executionStage)

		currentStage = executionStage
		height = stage.height
		width = stage.width

		content.currentIndex = stage.pageIndex
		var currentPage = content.children[content.currentIndex]
		if (currentPage.hasOwnProperty("realHeight"))
		{
			/// Костыль: не все страницы имеют фиксированную высоту (при вёрстке забыли про длинные локали).
			/// Чтобы всё не перевёрстывать, получаем тут реальную высоту страницы и перезаписываем.
			height = Math.round(currentPage.realHeight())
		}

		if (stage.activate)
			stage.activate()

		callback.AdjustViewSize()
	}

	function onNPSSubmitted() {
		nag.userNPS = ratePage.userScore
		activatePage(ratePage.userScore < lowerGoodRating
			? ExecutionStage.enterBadComment
			: (availableSites.length > 0 ? ExecutionStage.displaySiteButtons : ExecutionStage.enterGoodComment))
	}

	function onFeedbackSubmitted(feedback, email) {
		nag.userFeedback = feedback
		nag.userEmail = email
		activatePage(ExecutionStage.finalPage)
		sendFeedbackAndNPS()
	}

	function sendFeedbackAndNPS() {
		nag.isFeedbackAndNPSSent = true
		callback.RequestCustomAction(
			"sendNPSInfo",
			{
				"nps": nag.userNPS,
				"comment": nag.userFeedback,
				"email": nag.userEmail
			}
		)
	}

	function sendFeedbackAndClose() {
		if (nag.isFeedbackAndNPSSent)
			return

		if (currentStage == ExecutionStage.chooseFeedback)
			callback.Reject()
		else {
			sendFeedbackAndNPS()
			callback.Accept()
		}
		nag.reset()
	}

	function reset() {
		nag.userNPS = 0
		nag.userFeedback = ""
		nag.userEmail = ""
		nag.isFeedbackAndNPSSent = false
		activatePage(ExecutionStage.chooseFeedback)
	}
}
